#!/bin/sh

#
# ftp-suse.sh - charge un package pour la SuSE par ftp.
#
# Usage : ftp-suse.sh [-s] <package>
#
# Description :
# ftp-suse.sh charge le(s) package(s) <package> sur le site ftp de la
# distribution SuSE. Cela permet de trouver rapidemant le package sur le
# site, en le cherchant dans tous les repertoires.
#
# Options :
# <package> designe le nom du package a charger. Ce nom servira de modele
# pour le nom du package a charger. Par exemple, 'bind' donnera comme
# packages 'kpa1/kcmbind.rpm', 'n1/bind.rpm', 'n1/bind8.rpm', 'n1/bind9.rpm',
# 'n1/bindutil.rpm', 'zq1/bind.spm', 'zq1/bind8.spm', 'zq1/bind9.spm' ou
# encore 'zq1/kcmbind.spm'.
# [-s] permet d'effectuer une recherche dans l'index pour avoir une
# description de chaque package trouve.
#
# Exemple :
# ./ftp-suse.sh bind
# cherche tous les packages dont le nom contient 'bind', affiche la liste
# des packages trouves et demande si il faut les charger ou non.a
# ./ftp-suse.sh -s bind
# affiche la description des packages dont le nom contient 'bind'.
#
# Fichiers :
# 	/tmp/INDEX.SuSE.root : le fichier contenant l'index de la racine
# sur le serveur FTP.
# 	/tmp/INDEX.SuSE : le fichier d'index de tous les packages et de leur
# descriptions, utilise avec l'option [-s]. Si ce fichier n'existe pas, il
# est construit, sinon rien n'est charge.
# 	/etc/SuSE-release : le fichier contanant la version de la SuSE
# enployee (voir section suivante).
#
# Forcer la version de la SuSE :
# Si le systeme utilise n'est pas la SuSE sur laquelle on souhaite installer
# les packages, on peut le forcer a fonctionner en changant dans le script
# la variable [Version] par le numero de la version de la SuSE souhaitee.
# Par exemple : Version="7.0"
#
# Requiere :
# ftp
#

FicIndexTotal="/tmp/INDEX.SuSE"
#Version="7.0"

#
# NE RIEN EDITER AU DELA DE CETTE LIGNE	
#

[ $# -lt 1 ] && echo >&2 "Usage : $0 [-s] <package>" && exit 1

[ -z "$Version" ] && Version=`grep "VERSION" /etc/SuSE-release | sed s/'VERSION = '//`
Version=7.1 # La plus vieille, la 7.0 n'etant plus chargable

#
# Un mirroir en France.
#

#Serveur="fr2.rpmfind.net"
#Repertoire="/linux/SuSE-Linux"

#
# Le site officile (souvent trop charge).
#

Serveur="ftp.suse.com"
Repertoire="/pub"

[ -z "$Version" ] && Version=`cat /etc/SuSE-release | grep VERSION | awk '{print $3}'`

#
# Option de recherche de la chaine de caracteres parmis les descriptions de
# tous les packages. Si le fichier $FicIndexTotal n'existe pas (fichier
# contenant les infos de tous les packages), on le construit.
#

if [ "$1" = "-s" ]
then
    if [ -f $FicIndexTotal ]
    then
        grep -i "$2" $FicIndexTotal
        exit 0
    else
        echo -n "Dowloading INDEX file"
        ftp -i -n $Serveur << EOF > /dev/null
user ftp $LOGNAME
prompt off
cd $Repertoire/suse/i386/$Version/suse
get INDEX /tmp/INDEX
EOF
        if [ -f /tmp/INDEX ]
        then
            ListeRep=`cut -d: -f1 /tmp/INDEX`
        else
            echo
            echo >&2 "Erreur : /tmp/INDEX n'existe pas"
            echo >&2 "Le serveur est sans doute surcharge - essayer plus tard."
            exit 1
        fi

        for rep in $ListeRep
        do
            echo -n "."
#echo "$rep"
            ftp -i -n $Serveur << EOF > /dev/null
user ftp $LOGNAME
prompt off
cd $Repertoire/suse/i386/$Version/suse/$rep
get INDEX.english /tmp/INDEX.english
EOF
        if [ -f /tmp/INDEX.english ]
        then
            cat /tmp/INDEX.english >> $FicIndexTotal
        else
            echo >&2 "Erreur : /tmp/INDEX n'existe pas"
            echo >&2 "Le serveur est sans doute surcharge - essayer plus tard."
            exit 1
        fi
        done
        echo
        grep -i "$2" $FicIndexTotal
        echo "See $FicIndexTotal file for more info."
        rm -f /tmp/INDEX /tmp/INDEX.english
        exit 0
    fi
fi

#
# Recuperation du fichier INDEX de la racine
#

if [ ! -f $FicIndexTotal.root ]
then
    echo -n "Downloading root index file $FicIndexTotal.root"
    (ftp -i -n $Serveur << EOF
user ftp $LOGNAME
prompt off
cd $Repertoire/suse/i386/$Version/suse
get INDEX $FicIndexTotal.root
EOF
) | while read ligne
    do
        echo -n "."
    done
echo " done."
fi

[ ! -f $FicIndexTotal.root ] && echo >&2 "Erreur : $FicIndexTotal.root n'existe pas" && echo >&2 "Le serveur est sans doute surcharge - essayer plus tard." && exit 1

#
# Les packages qui correspondent
#

echo -n "Searching $1"
(
cat << EOF
user ftp $LOGNAME
prompt off
cd $Repertoire/suse/i386/$Version/suse
EOF

#for rep in `cut -d: -f1 INDEX`
for rep in `cut -d: -f1 $FicIndexTotal.root`
do
    echo "ls $rep/*$1*"
done
) | ftp -i -n $Serveur | tee /tmp/ftp.$$ | \
    while read ligne
    do
        echo -n "."
    done || (echo ; echo >&2 "Le serveur est sans doute surcharge - essayer plus tard." && exit 1)


echo " done."

ListeRPM=`cat /tmp/ftp.$$`
rm -f /tmp/ftp.$$

[ -z "$ListeRPM" ] && echo >&2 "$1: no match found." && exit 0

#
# Faut-il charger le(s) package(s) trouve(s) ?
#

for package in $ListeRPM
do
    echo "$package" | egrep ".rpm$|.spm$" > /dev/null || continue
    echo -n "Download $package ([y]/n)? "
    read reponse
    if [ "$reponse" = "y" -o "$reponse" = "" ]
    then
        ftp -i -n $Serveur << EOF
user ftp $LOGNAME
prompt off
bin
cd $Repertoire/suse/i386/$Version/suse/`dirname $package`
get `basename $package`
EOF
    fi
done
